const {Constants: Permission} = require("eris")

const utils = require("../utils/utils")

const channelInfos = msg => {
    return {
        embed: {
            title: "Channel infos",
            type: "rich",
            fields: [
                {name: "channel name", value: msg.channel.name, inline: true},
                {name: "mention", value: msg.channel.mention, inline: true},
                {name: "topic", value: msg.channel.topic && msg.channel.topic.length ? msg.channel.topic : "no topic set", inline: false},
                {
                    name: "type",
                    value: (()=>{
                        switch(msg.channel.type) {
                            case 0: return "Text channel"
                            case 2: return "Voice channel"
                            case 4: return "Category channel"
                            default: return msg.channel.type
                        }
                    })(),
                    inline: true
                },
                {name: "is nsfw", value: msg.channel.nsfw, inline: true}
            ]
        }
    }
}

const createChannel = (bot, msg, args) => {
    if (args.length === 0) {
        return "You need to specify a channel name."
    }

    const category = utils.getChannel(bot, msg.channel.parentID)
    if (category.channels.filter(channel => channel.type === 0).length >= 5) {
        return "You can't have more than 5 text channel in a category."
    }

    utils.getGuild(bot).createChannel(args.join` `, 0, {
        parentID: msg.channel.parentID
    })
    .then(channel => msg.channel.createMessage(`Success ${channel.mention}`))
    .catch(e => {
        msg.channel.createMessage(`An error happened, please contact <@${utils.ids.botDev}>:
${e.stack}`)
    })
}

const deleteChannel = (bot, msg, args) => {
    return "WIP"
    /*
    Please enter the confirmation number to continue 3437
     */
}

const configCommands = {
    name: async (bot, msg, args) => {
        await msg.channel.edit({name: args.join` `})
        return "Success"
    },
    topic: async (bot, msg, args) => {
        await msg.channel.edit({topic: args.join` `})
        return "Success"
    },
    visibility: async (bot, msg, args) => {
        return "WIP"
    },
    default: () => {
        return getConfigChannelOptionsString()
    }
}
function getConfigChannelOptionsString() {
    return "Config name is not valid, please use one of `" + Object.getOwnPropertyNames(configCommands).join("`, `") + "`"
}

module.exports = bot => {
    const channelCommand = bot.registerCommand("channel", channelInfos, {
        description: "Manage channels",
        fullDescription: "This command can be used to manage channels."
    })
    channelCommand.registerSubcommand("info", channelInfos, {
        aliases: ["infos", "i"],
        description: "Give info for current channel",
        fullDescription: "This command gives information about the current channel."
    })
    channelCommand.registerSubcommand("create", (msg, args) => createChannel(bot, msg, args), {
        aliases: ["new", "mk"],
        description: "Create a new channel",
        fullDescription: "This command allows leaders to create a new channel in the current category.",
        requirements: {
            custom:
                async msg =>
                    (
                        msg.member.roles.includes(utils.ids.roles.teamLead) &&
                        await global.database.categoryRoleInstance.getsByCategory(msg.channel.parentID)
                            .then(res => res
                                .map(el => el.roleId)
                                .filter(el => msg.member.roles.includes(el))
                                .length !== 0
                            )
                    ) ||
                    msg.channel.permissionsOf(msg.member.id).has("administrator")
        }
    })
    channelCommand.registerSubcommand("delete", (msg, args) => deleteChannel(bot, msg, args), {
        aliases: ["rm", "remove"],
        description: "Delete current channel ",
        fullDescription: "This command allows leader to delete the current channel.\n⚠️️️️️️️️️ this is not reversible ⚠️️️️️️️️️",
        requirements: {
            custom:
                async msg =>
                    (
                        msg.member.roles.includes(utils.ids.roles.teamLead) &&
                        await global.database.categoryRoleInstance.getsByCategory(msg.channel.parentID)
                            .then(res => res
                                .map(el => el.roleId)
                                .filter(el => msg.member.roles.includes(el))
                                .length !== 0
                            )
                    ) ||
                    msg.channel.permissionsOf(msg.member.id).has("administrator")
        }
    })
    const configCommand = channelCommand.registerSubcommand("config", (msg, args) => configCommands.default(), {
        description: "Configure current channel",
        fullDescription: "This command allows leader to configure the current channel.️️️️️️️️️",
        requirements: {
            custom:
                async msg =>
                    (
                        msg.member.roles.includes(utils.ids.roles.teamLead) &&
                        await global.database.categoryRoleInstance.getsByCategory(msg.channel.parentID)
                            .then(res => res
                                .map(el => el.roleId)
                                .filter(el => msg.member.roles.includes(el))
                                .length !== 0
                            )
                    ) ||
                    msg.channel.permissionsOf(msg.member.id).has("administrator")
        }
    })
    configCommand.registerSubcommand("name", (msg, args) => configCommands.name(bot, msg, args))
    configCommand.registerSubcommand("topic", (msg, args) => configCommands.topic(bot, msg, args), {aliases: ["desc", "description"]})
    configCommand.registerSubcommand("visibility", (msg, args) => configCommands.visibility(bot, msg, args), {aliases: ["vis"]})
}
