const fs = require("fs")
const DEFAULT_DB_FILE = __dirname + "/../../db.db"
const sqlite3 = require('sqlite-async')
const BotInstance = require("../database/bot")

class Database {
	async init(dbFile = DEFAULT_DB_FILE) {
		const exists = fs.existsSync(dbFile)
		await sqlite3.open(dbFile).then(async db => {
			this.db = db
			this.botInstance = new BotInstance(this.db)
			if(!exists) {
				await this.botInstance.updates[0]()
			}
			this.upgrade()
		})
	}

	upgrade() {
		this.db.get("SELECT version FROM bot").then(async ({version}) => {
			let newVersion = version

			Object.entries(this.botInstance.updates).forEach(entry => {
				if (+entry[0] > version) {
					entry[1]()
					newVersion = Math.max(newVersion, +entry[0])
				}
			})

			const updates = fs.readdirSync(__dirname + "/../database/")
				.filter(filename => filename !== "abstractInstance.js" && filename !== "bot.js" && filename.endsWith(".js"))
				.map(filename => filename.substr(0, filename.length - 3))
				.map(filenameNoExt => ({filenameNoExt, instanceClass: require(__dirname + "/../database/" + filenameNoExt)}))
				.map(InstanceInfos => {
					const Instance = new InstanceInfos.instanceClass(this.db)
					this[InstanceInfos.filenameNoExt + "Instance"] = Instance
					return Instance
				})
				.map(instance => instance.updates)

			const maxVersion = updates
				.map(update => Object.keys(update).reduce((acc, val) => Math.max(acc, +val), 0))
				.reduce((acc, val) => Math.max(acc, val))

			for (let i = version + 1; i <= maxVersion; i++) {
				await Promise.all(
					updates.map(update => update[i])
						.filter(update => Boolean(update))
						.map(update => update())
				)
			}

			newVersion = Math.max(newVersion, maxVersion)

			if (newVersion !== version) {
				this.botInstance.update({version: newVersion}).then(() => {
					console.log("Successfully updated database to version", newVersion)
				})
			} else {
				console.log("No database update required")
			}
		})
	}
}

module.exports = new Database()
